from __future__ import annotations

from enum import Enum


class CapabilityStepType(Enum):
    """
    Enum that specifies the types of CapabilitySteps that are possible
    """

    PrepareAndRunWorkflow = 0
    AwaitQA = 1
    AwaitWorkflow = 2
    AwaitProduct = 3
    AwaitParameter = 4
    AwaitLargeAllocApproval = 5
    Invalid = -1

    @classmethod
    def from_string(cls, string: str) -> CapabilityStepType:
        """
        Get capability step given a string

        :param string:
        :return: Capability step
        """
        strings = {
            "prepare-and-run-workflow": cls.PrepareAndRunWorkflow,
            "await-qa": cls.AwaitQA,
            "await-workflow": cls.AwaitWorkflow,
            "await-product": cls.AwaitProduct,
            "await-parameter": cls.AwaitParameter,
            "await-large-alloc-approval": cls.AwaitLargeAllocApproval,
        }
        return strings.get(string, cls.Invalid)


class CapabilityEventType(Enum):
    """
    Enum that specifies the type of CapabilityEvents possible to send
    """

    WorkflowComplete = 0
    QaApproval = 1
    QaDenial = 1
    WorkflowReady = 2
    ProductReady = 3
    ParameterReady = 4
    LargeAllocApproval = 5
    LargeAllocDenial = 5


class CapabilityRequestState(Enum):
    """
    Enum that specifies the states that a capability request can be in
    """

    Complete = 0
    Executing = 1
    Ready = 2
    Cancelled = 3


class ExecutionState(Enum):
    """
    Enum that specifies the states that a capability execution can find itself in
    """

    Complete = 0
    ExecutingStep = 1
    Ready = 2
    Cancelled = 3
    Failed = -1


class ExecutionPriority(Enum):
    """
    Enum that specifies the priority of a particular capability execution
    """

    High = 25
    Default = 50
    Low = 100
